mod power;
mod lib8;
mod render;
mod task;
mod time;
mod geometry;
mod platform;
mod animations;
mod mappings;
mod buffers;

use crate::platform::{DefaultBoard, Board};
use crate::task::{FixedSizeScheduler, Scheduler};
use crate::render::{Surfaces, Renderer};
use crate::geometry::Rectangle;

fn main() {
    let mut board: DefaultBoard = Board::take();

    log::info!("Board: {}", core::any::type_name_of_val(&board));

    let output = board.output();
    log::info!("Output: {}", core::any::type_name_of_val(&output));

    let mut surfaces = board.surfaces();
    log::info!("Surface implementation: {}", core::any::type_name_of_val(&output));

    log::info!("Creating animations");
    let mut animations = FixedSizeScheduler::new([
        Box::new(animations::IdleTask::new(&mut surfaces)),
        Box::new(animations::TestPattern::new(surfaces.new_surface(&Rectangle::everything()).unwrap())),
    ];

    let mut renderer = FixedSizeScheduler::new([Box::new(Renderer::new(output, surfaces))]);

    log::info!("Ready to rock and roll");
    loop {
        animations.tick();
        system.tick();
        renderer.tick();
    }
}
